//
//   Plik "main.c"
//
//   KURS AVR-GCC (abxyz.bplaced.net)
// 
//   Odczyt gamepada playstation 2 
// 
//   (opis w artykule)
//   testowanie na atmega8 (8MHz)
//
#include <avr/io.h>
#include <util/delay.h>


// Przypisujemy sygnay z gamepada do portw we/wa AVRa
// DATA - wejcie 
#define DATA_DIR_IN DDRB &= ~(1<<4)
#define DATA_IN    (PINB&(1<<4))
// CMD - wyjcie
#define CMD_SET PORTB |= (1<<3)
#define CMD_CLR PORTB &= ~(1<<3)
#define CMD_DIR_OUT DDRB |= (1<<3)
// ATT - wyjcie
#define ATT_SET PORTB |= (1<<2)
#define ATT_CLR PORTB &= ~(1<<2)
#define ATT_DIR_OUT DDRB |= (1<<2)
// CLK - wyjcie
#define CLK_SET PORTB |= (1<<5)
#define CLK_CLR PORTB &= ~(1<<5)
#define CLK_DIR_OUT DDRB |= (1<<5)

// Krotkie opnienie
#define _NOP_ asm volatile("nop\n\t""nop\n\t" "nop\n\t" "nop\n\t" ::)

// Wykorzystanie sprztowego SPI  
//#define ATMEGA_SPI 1

//---------------------------------------------------------------------------
// 9 bajtw danych odczytanych/zapisanyc z/do gamepada
unsigned char tab[9];

//---------------------------------------------------------------------------
// Konfiguruje porty we/wy i  sprztowy interfejs SPI
void init(void)
{
    // wyjcia - 8 diod LED
#define LEDS PORTD
    DDRD = 0xff;

    // wejcia - wybr  bajtu
#define  SWITCH (PINC)
    DDRC = 0x00;
    PORTC = 0x0f;

    CMD_DIR_OUT;
    ATT_DIR_OUT;
    CLK_DIR_OUT;

    ATT_SET;
    CLK_SET;
    CMD_SET;

#ifdef ATMEGA_SPI    
    SPCR =(1<<SPE)|(1<<MSTR)|(1<<DORD)|(1<<CPOL)|(1<<CPHA)|(1<<SPR0)|(1<<SPR1);
#endif
}

#ifdef ATMEGA_SPI    
//---------------------------------------------------------------------------
// Wysya/odczytuje jeden bajt do/z gamepada
// Funkcja korzysta ze sprztowego interfejsu SPI AVRa atmega 
unsigned char pad_byte(unsigned char byte)
{
    SPDR = byte;
    while(!(SPSR & (1<<SPIF)));
 
    _delay_us(20);
    return SPDR; 
}
#else
//---------------------------------------------------------------------------
// Wysya/odczytuje jeden bajt do/z gamepada
unsigned char pad_byte(unsigned char byte)
{
   unsigned char i,r=0;

   for(i=0; i<8; i++, byte>>=1)
   {
       CLK_CLR;
       if(byte& 0x01) CMD_SET; else CMD_CLR;
       _NOP_;
       CLK_SET;
       r>>=1;
       if(DATA_IN) r|=0x80;
   }
   CMD_SET;
   _delay_us(20);
   return r;   
}
#endif

//---------------------------------------------------------------------------
// Wysya/odbiera  do/z gamepada n bajtw 
void pad_cmd(unsigned char t[],unsigned char n)
{
   unsigned char i;

   ATT_CLR;
 
   for(i=0; i<n; i++)
      t[i] = pad_byte(t[i]);

   ATT_SET;
   _delay_us(50);
}


//---------------------------------------------------------------------------
// Komendy konfiguracyjne gamepada
void pad_config(void)
{
   // Komenda 0x43 "Go into configuration mode" 
   tab[0]= 0x01;
   tab[1]= 0x43;
   tab[2]= 0x00;
   tab[3]= 0x01;
   tab[4]= 0x00;
   pad_cmd(tab, 5);

   // Przeczamy gamepada w tryb analogowy, aby mie 
   // moliwo odczytywania dojstikw analogowych

   // Komenda 0x44 "Turn on analog mode" 
   tab[0]= 0x01;
   tab[1]= 0x44;
   tab[2]= 0x00;
   tab[3]= 0x01;
   tab[4]= 0x03; // 
   tab[5]= 0x00;
   tab[6]= 0x00;
   tab[7]= 0x00;
   tab[8]= 0x00;
   pad_cmd(tab, 9);

   // Komenda 0x43 "Exit config mode"
   tab[0] = 0x01; 
   tab[1] = 0x43; 
   tab[2] = 0x00;
   tab[3] = 0x00;
   tab[4] = 0x5A;
   tab[5] = 0x5A;
   tab[6] = 0x5A;
   tab[7] = 0x5A;
   tab[8] = 0x5A;
   pad_cmd(tab, 9);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//GOWNA FUNKCJA
int main(void)
{ 
   unsigned char n = 0;
   //  Konfiguruje porty we/wy 
   init();
   // Przeczamy w tryb analogowy 
   pad_config();

   // Gwna ptla programu
   while(1)
   {
      // odczyt przecznikw
      n = SWITCH & 0x0f;
      if(n>8) n = 0;
            
      // Nagwek pakietu
      tab[0] = 0x01;
      tab[1] = 0x42; // komenda 0x42 - Controller poll
      tab[2] = 0x00;

      tab[3] = 0x00; // przyciski
      tab[4] = 0x00; // przyciski
      tab[5] = 0x00; // dojstik prawy  lewo-prawo
      tab[6] = 0x00; // dojstik prawy  gra-d
      tab[7] = 0x00; // dojstik lewy   lewo-prawo
      tab[8] = 0x00; // dojstik lewy   gora-d

      // Wysyamy pakiet danych z komend 0x42(Controller poll),
      // w rezultacie otrzymujemy 9 bajtw z informacj o stanie 
      // przyciskw i dojstikw analogowych
      pad_cmd(tab, 9);
      
      // Wywietla na LEDach  wybrany bajt tablicy  
      switch(n)
      {
          case 0:
          case 1:
          case 2:
             LEDS = tab[n];
	  break;
                   // przyciski cyfrowe
          case 3: 
          case 4:
             LEDS = ~tab[n]; // warto bitu 0 oznacza wcinicie przycisku
          break;   
                   // dojstiki analogowe
          case 5: 
          case 6:
          case 7:
          case 8:
          case 9:
              LEDS = 1<<(tab[n]>>5); // wieci si jedna z omiu diod LED
          break;  
      }
      _delay_ms(50);
   } // koniec gwnej ptli 

   return 0;
}

//---------------------------------------------------------------------------
// KONIEC PLIKU
